@@ -5,7 +5,7 @@ use serde_json::Value;
 
 use crate::errors::Result;
 use crate::jsontypes::{RawSection, RawSectionOffset, RawSourceMap};
-use crate::types::{SourceMap, SourceMapIndex};
+use crate::types::{DecodedMap, SourceMap, SourceMapIndex};
 use crate::vlq::encode_vlq;
 
 pub trait Encodable {
@@ -19,7 +19,7 @@ pub fn encode<M: Encodable, W: Write>(sm: &M, mut w: W) -> Result<()> {
 }
 
 fn encode_vlq_diff(out: &mut String, a: u32, b: u32) {
-    encode_vlq(out, (a as i64) - (b as i64))
+    encode_vlq(out, i64::from(a) - i64::from(b))
 }
 
 fn serialize_mappings(sm: &SourceMap) -> String {
@@ -92,6 +92,8 @@ impl Encodable for SourceMap {
             sections: None,
             names: Some(self.names().map(|x| Value::String(x.to_string())).collect()),
             mappings: Some(serialize_mappings(self)),
+            x_facebook_offsets: None,
+            x_metro_module_paths: None,
         }
     }
 }
@@ -111,7 +113,7 @@ impl Encodable for SourceMapIndex {
                             line: section.get_offset_line(),
                             column: section.get_offset_col(),
                         },
-                        url: section.get_url().map(|x| x.to_string()),
+                        url: section.get_url().map(str::to_owned),
                         map: section
                             .get_sourcemap()
                             .map(|sm| Box::new(sm.as_raw_sourcemap())),
@@ -120,6 +122,17 @@ impl Encodable for SourceMapIndex {
             ),
             names: None,
             mappings: None,
+            x_facebook_offsets: None,
+            x_metro_module_paths: None,
+        }
+    }
+}
+
+impl Encodable for DecodedMap {
+    fn as_raw_sourcemap(&self) -> RawSourceMap {
+        match *self {
+            DecodedMap::Regular(ref sm) => sm.as_raw_sourcemap(),
+            DecodedMap::Index(ref smi) => smi.as_raw_sourcemap(),
         }
     }
 }
