@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::cmp::Ordering;
 use std::fmt;
 use std::io::{Read, Write};
@@ -77,7 +78,7 @@ impl DecodedMap {
     /// regular sourcemap.  This method can only be used if
     /// the contained index actually contains embedded maps
     /// or it will not be able to look up anything.
-    pub fn lookup_token<'a>(&'a self, line: u32, col: u32) -> Option<Token<'a>> {
+    pub fn lookup_token(&self, line: u32, col: u32) -> Option<Token<'_>> {
         match *self {
             DecodedMap::Regular(ref sm) => sm.lookup_token(line, col),
             DecodedMap::Index(ref smi) => smi.lookup_token(line, col),
@@ -251,6 +252,19 @@ pub struct TokenIter<'a> {
     next_idx: u32,
 }
 
+impl<'a> TokenIter<'a> {
+    pub fn seek(&mut self, line: u32, col: u32) -> bool {
+        let token = self.i.lookup_token(line, col);
+        match token {
+            Some(token) => {
+                self.next_idx = token.idx + 1;
+                true
+            }
+            None => false,
+        }
+    }
+}
+
 impl<'a> Iterator for TokenIter<'a> {
     type Item = Token<'a>;
 
@@ -349,7 +363,7 @@ impl<'a> fmt::Display for Token<'a> {
             self.get_src_col(),
             self.get_name()
                 .map(|x| format!(" name={}", x))
-                .unwrap_or("".into())
+                .unwrap_or_default()
         )?;
         if f.alternate() {
             write!(f, " ({}:{})", self.get_dst_line(), self.get_dst_col())?;
@@ -362,7 +376,7 @@ impl<'a> fmt::Display for Token<'a> {
 pub struct SourceMapSection {
     offset: (u32, u32),
     url: Option<String>,
-    map: Option<Box<SourceMap>>,
+    map: Option<Box<DecodedMap>>,
 }
 
 /// Iterates over all sections in a sourcemap index
@@ -386,6 +400,8 @@ impl<'a> Iterator for SourceMapSectionIter<'a> {
 pub struct SourceMapIndex {
     file: Option<String>,
     sections: Vec<SourceMapSection>,
+    x_facebook_offsets: Option<Vec<Option<u32>>>,
+    x_metro_module_paths: Option<Vec<String>>,
 }
 
 /// Represents a sourcemap in memory
@@ -393,6 +409,7 @@ pub struct SourceMapIndex {
 /// This is always represents a regular "non-indexed" sourcemap.  Particularly
 /// in case the `from_reader` method is used an index sourcemap will be
 /// rejected with an error on reading.
+#[derive(Clone, Debug)]
 pub struct SourceMap {
     file: Option<String>,
     tokens: Vec<RawToken>,
@@ -497,15 +514,15 @@ impl SourceMap {
             .collect();
         index.sort();
         SourceMap {
-            file: file,
-            tokens: tokens,
-            index: index,
-            names: names,
-            sources: sources,
+            file,
+            tokens,
+            index,
+            names,
+            sources,
             sources_content: sources_content
-                .unwrap_or(vec![])
+                .unwrap_or_default()
                 .into_iter()
-                .map(|opt| opt.map(|source| SourceView::from_string(source)))
+                .map(|opt| opt.map(SourceView::from_string))
                 .collect(),
         }
     }
@@ -517,16 +534,14 @@ impl SourceMap {
 
     /// Sets a new value for the file.
     pub fn set_file(&mut self, value: Option<&str>) {
-        self.file = value.map(|x| x.to_string());
+        self.file = value.map(str::to_owned);
     }
 
     /// Looks up a token by its index.
-    pub fn get_token<'a>(&'a self, idx: u32) -> Option<Token<'a>> {
-        self.tokens.get(idx as usize).map(|raw| Token {
-            raw: raw,
-            i: self,
-            idx: idx,
-        })
+    pub fn get_token(&self, idx: u32) -> Option<Token<'_>> {
+        self.tokens
+            .get(idx as usize)
+            .map(|raw| Token { raw, i: self, idx })
     }
 
     /// Returns the number of tokens in the sourcemap.
@@ -535,7 +550,7 @@ impl SourceMap {
     }
 
     /// Returns an iterator over the tokens.
-    pub fn tokens<'a>(&'a self) -> TokenIter<'a> {
+    pub fn tokens(&self) -> TokenIter<'_> {
         TokenIter {
             i: self,
             next_idx: 0,
@@ -543,7 +558,7 @@ impl SourceMap {
     }
 
     /// Looks up the closest token to a given line and column.
-    pub fn lookup_token<'a>(&'a self, line: u32, col: u32) -> Option<Token<'a>> {
+    pub fn lookup_token(&self, line: u32, col: u32) -> Option<Token<'_>> {
         let mut low = 0;
         let mut high = self.index.len();
 
@@ -602,7 +617,7 @@ impl SourceMap {
     }
 
     /// Iterates over all sources
-    pub fn sources<'a>(&'a self) -> SourceIter<'a> {
+    pub fn sources(&self) -> SourceIter<'_> {
         SourceIter {
             i: self,
             next_idx: 0,
@@ -610,18 +625,18 @@ impl SourceMap {
     }
 
     /// Returns the sources content as source view.
-    pub fn get_source_view<'a>(&'a self, idx: u32) -> Option<&'a SourceView<'a>> {
+    pub fn get_source_view(&self, idx: u32) -> Option<&SourceView<'_>> {
         self.sources_content
             .get(idx as usize)
-            .and_then(|x| x.as_ref())
+            .and_then(Option::as_ref)
     }
 
     /// Looks up the content for a source.
     pub fn get_source_contents(&self, idx: u32) -> Option<&str> {
         self.sources_content
             .get(idx as usize)
-            .and_then(|bucket| bucket.as_ref())
-            .map(|x| x.source())
+            .and_then(Option::as_ref)
+            .map(SourceView::source)
     }
 
     /// Sets source contents for a source.
@@ -633,7 +648,7 @@ impl SourceMap {
     }
 
     /// Iterates over all source contents
-    pub fn source_contents<'a>(&'a self) -> SourceContentsIter<'a> {
+    pub fn source_contents(&self) -> SourceContentsIter<'_> {
         SourceContentsIter {
             i: self,
             next_idx: 0,
@@ -641,7 +656,7 @@ impl SourceMap {
     }
 
     /// Returns an iterator over the names.
-    pub fn names<'a>(&'a self) -> NameIter<'a> {
+    pub fn names(&self) -> NameIter<'_> {
         NameIter {
             i: self,
             next_idx: 0,
@@ -674,7 +689,7 @@ impl SourceMap {
     }
 
     /// Returns the number of items in the index
-    pub fn index_iter<'a>(&'a self) -> IndexIter<'a> {
+    pub fn index_iter(&self) -> IndexIter<'_> {
         IndexIter {
             i: self,
             next_idx: 0,
@@ -729,7 +744,7 @@ impl SourceMap {
             }
         }
         if need_common_prefix {
-            if let Some(prefix) = find_common_prefix(self.sources.iter().map(|x| x.as_str())) {
+            if let Some(prefix) = find_common_prefix(self.sources.iter().map(String::as_str)) {
                 prefixes.push(prefix);
             }
         }
@@ -773,10 +788,17 @@ impl SourceMapIndex {
     ///
     /// - `file`: an optional filename of the index
     /// - `sections`: a vector of source map index sections
-    pub fn new(file: Option<String>, sections: Vec<SourceMapSection>) -> SourceMapIndex {
+    pub fn new(
+        file: Option<String>,
+        sections: Vec<SourceMapSection>,
+        x_facebook_offsets: Option<Vec<Option<u32>>>,
+        x_metro_module_paths: Option<Vec<String>>,
+    ) -> SourceMapIndex {
         SourceMapIndex {
-            file: file,
-            sections: sections,
+            file,
+            sections,
+            x_facebook_offsets,
+            x_metro_module_paths,
         }
     }
 
@@ -787,7 +809,7 @@ impl SourceMapIndex {
 
     /// Sets a new value for the file.
     pub fn set_file(&mut self, value: Option<&str>) {
-        self.file = value.map(|x| x.to_string());
+        self.file = value.map(str::to_owned);
     }
 
     /// Returns the number of sections in this index
@@ -806,7 +828,7 @@ impl SourceMapIndex {
     }
 
     /// Iterates over all sections
-    pub fn sections<'a>(&'a self) -> SourceMapSectionIter<'a> {
+    pub fn sections(&self) -> SourceMapSectionIter<'_> {
         SourceMapSectionIter {
             i: self,
             next_idx: 0,
@@ -818,9 +840,10 @@ impl SourceMapIndex {
     /// This requires that the referenced sourcemaps are actually loaded.
     /// If a sourcemap is encountered that is not embedded but just
     /// externally referenced it is silently skipped.
-    pub fn lookup_token<'a>(&'a self, line: u32, col: u32) -> Option<Token<'a>> {
+    pub fn lookup_token(&self, line: u32, col: u32) -> Option<Token<'_>> {
         for section in self.sections() {
             let (off_line, off_col) = section.get_offset();
+            println!("off_line: {}, off_col: {}", off_line, off_col);
             if off_line < line || off_col < col {
                 continue;
             }
@@ -835,13 +858,16 @@ impl SourceMapIndex {
 
     /// Flattens an indexed sourcemap into a regular one.  This requires
     /// that all referenced sourcemaps are attached.
-    pub fn flatten(self) -> Result<SourceMap> {
+    pub fn flatten(&self) -> Result<SourceMap> {
         let mut builder = SourceMapBuilder::new(self.get_file());
 
         for section in self.sections() {
             let (off_line, off_col) = section.get_offset();
             let map = match section.get_sourcemap() {
-                Some(map) => map,
+                Some(map) => match map {
+                    DecodedMap::Regular(sm) => Cow::Borrowed(sm),
+                    DecodedMap::Index(idx) => Cow::Owned(idx.flatten()?),
+                },
                 None => {
                     return Err(Error::CannotFlatten(format!(
                         "Section has an unresolved \
@@ -878,6 +904,18 @@ impl SourceMapIndex {
     pub fn flatten_and_rewrite(self, options: &RewriteOptions<'_>) -> Result<SourceMap> {
         self.flatten()?.rewrite(options)
     }
+
+    pub fn is_for_react_native(&self) -> bool {
+        self.x_facebook_offsets.is_some() && self.x_metro_module_paths.is_some()
+    }
+
+    pub fn x_facebook_offsets(&self) -> Option<&Vec<Option<u32>>> {
+        self.x_facebook_offsets.as_ref()
+    }
+
+    pub fn x_metro_module_paths(&self) -> Option<&Vec<String>> {
+        self.x_metro_module_paths.as_ref()
+    }
 }
 
 impl SourceMapSection {
@@ -889,12 +927,12 @@ impl SourceMapSection {
     pub fn new(
         offset: (u32, u32),
         url: Option<String>,
-        map: Option<SourceMap>,
+        map: Option<DecodedMap>,
     ) -> SourceMapSection {
         SourceMapSection {
-            offset: offset,
-            url: url,
-            map: map.map(|x| Box::new(x)),
+            offset,
+            url,
+            map: map.map(Box::new),
         }
     }
 
@@ -920,21 +958,21 @@ impl SourceMapSection {
 
     /// Updates the URL for this section.
     pub fn set_url(&mut self, value: Option<&str>) {
-        self.url = value.map(|x| x.to_string());
+        self.url = value.map(str::to_owned);
     }
 
     /// Returns a reference to the embedded sourcemap if available
-    pub fn get_sourcemap(&self) -> Option<&SourceMap> {
-        self.map.as_ref().map(|x| &**x)
+    pub fn get_sourcemap(&self) -> Option<&DecodedMap> {
+        self.map.as_ref().map(Box::as_ref)
     }
 
     /// Returns a reference to the embedded sourcemap if available
-    pub fn get_sourcemap_mut(&mut self) -> Option<&mut SourceMap> {
-        self.map.as_mut().map(|x| &mut **x)
+    pub fn get_sourcemap_mut(&mut self) -> Option<&mut DecodedMap> {
+        self.map.as_mut().map(Box::as_mut)
     }
 
     /// Replaces the embedded sourcemap
-    pub fn set_sourcemap(&mut self, sm: Option<SourceMap>) {
-        self.map = sm.map(|x| Box::new(x));
+    pub fn set_sourcemap(&mut self, sm: Option<DecodedMap>) {
+        self.map = sm.map(Box::new);
     }
 }
